SELECT
    column_name,
    data_type
    {%- if include_table_names -%},
    table_name --noqa: LT02
    {%- endif %}
FROM information_schema.columns
WHERE
    table_schema = '{{ schema_name }}'
    AND table_name IN ('{{ table_names|join("', '")|lower }}')
{%- if column_names %}
    AND LOWER(column_name) IN ('{{ column_names|join("', '")|lower }}') --noqa: LT02,LT05
{%- endif %}
